@@ -5,7 +5,7 @@ use warnings;
 use parent 'Perl::Critic::Policy';
 use Perl::Critic::Utils ':severities';
 
-$Perl::Critic::Policy::Variables::RequireHungarianNotation::VERSION = '0.0.1';
+$Perl::Critic::Policy::Variables::RequireHungarianNotation::VERSION = '0.0.2';
 
 sub desc {
     return 'Non hungarian notation';
@@ -17,17 +17,18 @@ sub expl {
 
 sub supported_parameters {
     return (
-        {name => 'array',     description => 'Prefix for Hash vars',                      default_string => q/a_/,},
-        {name => 'hash',      description => 'Prefix for Array vars',                     default_string => q/h_/,},
-        {name => 'array_ref', description => 'Prefix for Arrayref vars',                  default_string => q/ar_/,},
-        {name => 'hash_ref',  description => 'Prefix for Hashref vars',                   default_string => q/hr_/,},
-        {name => 'object',    description => 'Prefix for Object vars',                    default_string => q/o_/,},
-        {name => 'integer',   description => 'Prefix for Integer vars',                   default_string => q/i_/,},
-        {name => 'string',    description => 'Prefix for String vars',                    default_string => q/s_/,},
-        {name => 'boolean',   description => 'Prefix for boolean vars',                   default_string => q/b_/},
-        {name => 'regex',     description => 'Prefix for predefined regular expressions', default_string => q/rx_/},
-        {name => 'self',      description => 'Ingore self variable',                      default_string => q/self/},
-        {name => 'custom',    description => 'Custom prefixes',},
+        { name => 'array',     description => 'Prefix for Hash vars',                      default_string => q/a_/, },
+        { name => 'hash',      description => 'Prefix for Array vars',                     default_string => q/h_/, },
+        { name => 'array_ref', description => 'Prefix for Arrayref vars',                  default_string => q/ar_/, },
+        { name => 'hash_ref',  description => 'Prefix for Hashref vars',                   default_string => q/hr_/, },
+        { name => 'code_ref',  description => 'Prefix for Hashref vars',                   default_string => q/cr_/, },
+        { name => 'object',    description => 'Prefix for Object vars',                    default_string => q/o_/, },
+        { name => 'integer',   description => 'Prefix for Integer vars',                   default_string => q/i_/, },
+        { name => 'string',    description => 'Prefix for String vars',                    default_string => q/s_/, },
+        { name => 'boolean',   description => 'Prefix for boolean vars',                   default_string => q/b_/ },
+        { name => 'regex',     description => 'Prefix for predefined regular expressions', default_string => q/rx_/ },
+        { name => 'self',      description => 'Ingore self variable',                      default_string => q/self/ },
+        { name => 'custom',    description => 'Custom prefixes', },
     );
 }
 
@@ -44,45 +45,30 @@ sub applies_to {
 }
 
 sub violates {
-    my ($self, $o_elem, undef) = @_;
-    return if not scalar grep { $o_elem->type eq $_ } qw/my our local/;
-    if ($self->_violates_scalar($o_elem) || $self->_violates_list($o_elem)) {
-        return $self->violation(desc(), expl(), $o_elem);
+    my ( $self, $o_variable, undef ) = @_;
+    return if not scalar grep { $o_variable->type eq $_ } qw/my our local/;
+    if ( $self->_violates_scalar($o_variable) || $self->_violates_list($o_variable) ) {
+        return $self->violation( desc(), expl(), $o_variable );
     }
+    return $self->violation( desc(), expl(), $o_variable ) if $self->_check_sub_variables($o_variable);
     return;
 }
 
 sub _violates_list {
-    my ($self, $o_elem) = @_;
-    my @a_significant_children = $o_elem->schildren;    # "significant" children
-    if ($a_significant_children[1]->isa('PPI::Token::Symbol') && $a_significant_children[1]->raw_type =~ /^[@%]$/) {
-        my $s_symbol   = $a_significant_children[1]->symbol;
-        my $s_raw_type = $a_significant_children[1]->raw_type;
-        for (qw/array hash/) {
-            return if $self->_check_prefix($s_symbol, $s_raw_type, $_);
-        }
-        return 1;
+    my ( $self, $o_variable ) = @_;
+    my @a_childrens = $o_variable->schildren;    # "significant" children
+    if ( $a_childrens[1]->isa('PPI::Token::Symbol') && $a_childrens[1]->raw_type =~ /^[@%]$/ ) {
+        return 1 if !$self->_b_check_list( $a_childrens[1]->symbol, $a_childrens[1]->raw_type );
     }
     return;
 }
 
 sub _violates_scalar {
-    my ($self, $o_elem) = @_;
-    my @a_significant_children = $o_elem->schildren;    # "significant" children
-
-    if ($a_significant_children[1]->isa('PPI::Token::Symbol') && $a_significant_children[1]->raw_type eq q{$}) {
-        my $s_symbol = $a_significant_children[1]->symbol;
-        return if $s_symbol eq q/$/ . $self->{_self};
-        for (qw/array_ref hash_ref object integer string boolean regex/) {
-            return if $self->_check_prefix($s_symbol, q/$/, $_);
-        }
-        if ($self->{_custom}) {
-            my $s_custom_prefixes = $self->_get_custom_prefixes;
-            return if $self->_check_prefix($s_symbol, q/$/, qq/($s_custom_prefixes)/);
-        }
-        return 1;
+    my ( $self, $o_variable ) = @_;
+    my @a_childrens = $o_variable->schildren;    # "significant" children
+    if ( $a_childrens[1]->isa('PPI::Token::Symbol') && $a_childrens[1]->raw_type eq q{$} ) {
+        return 1 if !$self->_b_check_scalar( $a_childrens[1]->symbol );
     }
-
     return;
 }
 
@@ -91,9 +77,47 @@ sub _get_custom_prefixes {
     return join q/|/, split / /, $self->{_custom};
 }
 
+sub _check_sub_variables {
+    my ( $self, $o_variable ) = @_;
+    return if not $o_variable->variables > 1;
+
+    for ( map { $_->schildren } grep { $_->isa('PPI::Structure::List') } $o_variable->schildren ) {
+        for ( grep { $_->isa('PPI::Token::Symbol') } $_->schildren ) {
+            return 1
+              if !(
+                  $_->raw_type =~ /^[@%]$/
+                ? $self->_b_check_list( $_->symbol, $_->raw_type )
+                : $self->_b_check_scalar( $_->symbol, $_->raw_type )
+              );
+        }
+    }
+    return;
+}
+
+sub _b_check_scalar {
+    my ( $self, $s_symbol, $s_raw_type ) = @_;
+    return 1 if $s_symbol eq q/$/ . $self->{_self};
+    for (qw/array_ref hash_ref object integer string boolean regex/) {
+        return 1 if $self->_check_prefix( $s_symbol, q/$/, $_ );
+    }
+    if ( $self->{_custom} ) {
+        my $s_custom_prefixes = $self->_get_custom_prefixes;
+        return 1 if $self->_check_prefix( $s_symbol, q/$/, qq/($s_custom_prefixes)/ );
+    }
+    return 0;
+}
+
+sub _b_check_list {
+    my ( $self, $s_symbol, $s_raw_type ) = @_;
+    for (qw/array hash/) {
+        return 1 if $self->_check_prefix( $s_symbol, $s_raw_type, $_ );
+    }
+    return 0;
+}
+
 sub _check_prefix {
-    my ($self, $s_symbol, $s_raw_type, $s_key) = @_;
-    my $s_rx_scalar = q/\\/ . $s_raw_type . ($self->{q/_/ . $s_key} // $s_key);
+    my ( $self, $s_symbol, $s_raw_type, $s_key ) = @_;
+    my $s_rx_scalar = q/\\/ . $s_raw_type . ( $self->{ q/_/ . $s_key } // $s_key );
     return $s_symbol =~ /^$s_rx_scalar/ ? 1 : 0;
 }
 
@@ -106,7 +130,7 @@ Perl::Critic::Policy::Variables::RequireHungarianNotation - prohibit hungarian n
 
 =head1 VERSION
 
-Version 0.0.1
+Version 0.0.2
 
 =head1 SYNOPSIS
 
@@ -154,22 +178,6 @@ Method to determine if the element currently under scrutiny violates this
 policy.  If it does, return a properly constructed C<Perl::Critic::Violation>
 object.  Otherwise, return C<undef>.
 
-=head2 violates_scalar($o_elem)
-
-Returns true if C<$o_elem> contains an assignment of the form
-
-    my $foo = undef;
-
-See L<http://search.cpan.org/dist/PPI/lib/PPI/Statement/Variable.pm> for
-details on how this function works.
-
-=head2 violates_list($o_elem)
-
-Returns true if C<$o_elem> contains an assignment of the forms:
-
-    my @foo = ();   # useless array init
-    my %bar = ();   # useless hash init
-
 =head2 desc()
 
 =head1 INTERFACE 
@@ -228,7 +236,7 @@ Mario Zieschang  C<< <mziescha -at - cpan -dot- org> >>
 Copyright (c) 2018, Mario Zieschang C<< <mziescha@cpan.org> >>. All rights reserved.
 
 This module is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself. See L<perlartistic>.
+modify it under the same terms as Perl itself. See L<perlartistic|perlartistic>.
 
 
 =head1 DISCLAIMER OF WARRANTY
